# Copyright 2015-2018 David Hadka
#
# This file is part of Platypus, a Python module for designing and using
# evolutionary algorithms (EAs) and multiobjective evolutionary algorithms
# (MOEAs).
#
# Platypus is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Platypus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Platypus.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import, division, print_function

import six
from .types import Real, Binary, Permutation, Subset
from .operators import GAOperator, CompoundOperator, CompoundMutation, SBX, PM, HUX, BitFlip, PMX, Insertion, Swap, SSX, Replace
from .core import PlatypusError
from .evaluator import MapEvaluator

class _PlatypusConfig(object):
    
    def __init__(self):
        super(_PlatypusConfig, self).__init__()
    
        self.default_variator = {Real : GAOperator(SBX(), PM()),
                                 Binary : GAOperator(HUX(), BitFlip()),
                                 Permutation : CompoundOperator(PMX(), Insertion(), Swap()),
                                 Subset : GAOperator(SSX(), Replace())}
        
        self.default_mutator = {Real : PM(),
                                Binary : BitFlip(),
                                Permutation : CompoundMutation(Insertion(), Swap()),
                                Subset : Replace()}
        
        self.default_evaluator = MapEvaluator()
        
        self.default_log_frequency = None
        
PlatypusConfig = _PlatypusConfig()
    
def default_variator(problem):
    if len(problem.types) == 0:
        raise PlatypusError("problem has no decision variables")
    
    base_type = problem.types[0].__class__
    
    if all([isinstance(t, base_type) for t in problem.types]):
        if base_type in PlatypusConfig.default_variator:
            return PlatypusConfig.default_variator[base_type]
        else:
            for default_type in six.iterkeys(PlatypusConfig.default_variator):
                if issubclass(base_type, default_type):
                    return PlatypusConfig.default_variator[default_type]
                
            raise PlatypusError("no default variator for %s" % base_type)
    else:
        raise PlatypusError("must define variator for mixed types")
    
def default_mutator(problem):
    if len(problem.types) == 0:
        raise PlatypusError("problem has no decision variables")
    
    base_type = problem.types[0].__class__
    
    if all([isinstance(t, base_type) for t in problem.types]):
        if base_type in PlatypusConfig.default_mutator:
            return PlatypusConfig.default_mutator[base_type]
        else:
            for default_type in six.iterkeys(PlatypusConfig.default_mutator):
                if issubclass(base_type, default_type):
                    return PlatypusConfig.default_mutator[default_type]
                
            raise PlatypusError("no default mutator for %s" % base_type)
    else:
        raise PlatypusError("must define mutator for mixed types")
    


#!/usr/bin/env python

from setuptools import setup
from setuptools.command.test import test as TestCommand

class NoseTestCommand(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import nose
        nose.run_exit(argv=['nosetests'])

setup(name='Platypus-Opt',
      version='1.0.0', # Update __init__.py if the version changes!
      description='Multiobjective optimization in Python',
      author='David Hadka',
      author_email='dhadka@users.noreply.github.com',
      license="GNU GPL version 3",
      url='https://github.com/Project-Platypus/Platypus',
      packages=['platypus'],
      install_requires=['six'],
      tests_require=['nose', 'mock'],
      cmdclass={'test': NoseTestCommand},
     )


# Copyright 2015-2018 David Hadka
#
# This file is part of Platypus, a Python module for designing and using
# evolutionary algorithms (EAs) and multiobjective evolutionary algorithms
# (MOEAs).
#
# Platypus is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Platypus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Platypus.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import, division, print_function

from .core import *
from .algorithms import *
from .evaluator import *
from .experimenter import *
from .indicators import *
from .operators import *
from .problems import *
from .tools import *
from .types import *
from .weights import *

__version__ = "1.0.0" # Update setup.py if the version changes!


# Copyright 2015-2018 David Hadka
#
# This file is part of Platypus, a Python module for designing and using
# evolutionary algorithms (EAs) and multiobjective evolutionary algorithms
# (MOEAs).
#
# Platypus is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Platypus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Platypus.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import, division, print_function

import math
import copy
import random
from .core import POSITIVE_INFINITY

def chebyshev(values, weights, min_weight=0.0001):
    return max([max(weights[i], min_weight) * values[i] for i in range(len(values))])

def random_weights(size, nobjs):
    weights = []
    
    if nobjs == 2:
        weights = [[1, 0], [0, 1]]
        weights.extend([(i/(size-1.0), 1.0-i/(size-1.0)) for i in range(1, size-1)])
    else:
        # generate candidate weights
        candidate_weights = []
        
        for i in range(size*50):
            random_values = [random.uniform(0.0, 1.0) for _ in range(nobjs)]
            candidate_weights.append([x/sum(random_values) for x in random_values])
        
        # add weights for the corners
        for i in range(nobjs):
            weights.append([0]*i + [1] + [0]*(nobjs-i-1))
            
        # iteratively fill in the remaining weights by finding the candidate
        # weight with the largest distance from the assigned weights
        while len(weights) < size:
            max_index = -1
            max_distance = -POSITIVE_INFINITY
            
            for i in range(len(candidate_weights)):
                distance = POSITIVE_INFINITY
                
                for j in range(len(weights)):
                    temp = math.sqrt(sum([math.pow(candidate_weights[i][k]-weights[j][k], 2.0) for k in range(nobjs)]))
                    distance = min(distance, temp)
                    
                if distance > max_distance:
                    max_index = i
                    max_distance = distance
                    
            weights.append(candidate_weights[max_index])
            del candidate_weights[max_index]
            
    return weights

def normal_boundary_weights(nobjs, divisions_outer, divisions_inner=0):
    def generate_recursive(weights, weight, left, total, index):
        if index == nobjs - 1:
            weight[index] = float(left) / float(total)
            weights.append(copy.copy(weight))
        else:
            for i in range(left+1):
                weight[index] = float(i) / float(total)
                generate_recursive(weights, weight, left-i, total, index+1)
    
    def generate_weights(divisions):
        weights = []
        generate_recursive(weights, [0.0]*nobjs, divisions, divisions, 0)
        return weights
        
    weights = generate_weights(divisions_outer)
    
    if divisions_inner > 0:
        inner_weights = generate_weights(divisions_inner)
        
        for i in range(len(inner_weights)):
            weight = inner_weights[i]
            
            for j in range(len(weight)):
                weight[j] = (1.0 / nobjs + weight[j]) / 2.0
                
            weights.append(weight)
        
    return weights